@@ -4,15 +4,11 @@ from typing import (
     TypeVar,
     Any,
     Type,
-    get_type_hints,
-    Callable,
     Iterator,
     Union,
 )
 
 from typing_extensions import ParamSpec
-import inspect
-from functools import wraps
 from contextlib import suppress
 import faulthandler
 from io import UnsupportedOperation
@@ -23,7 +19,7 @@ with suppress(
 ):  # in case its running in idle or something like that
     faulthandler.enable()
 
-__all__ = ("dereference_address", "Pointer", "to_ptr", "decay")
+__all__ = ("dereference_address", "Pointer", "to_ptr")
 
 T = TypeVar("T")
 A = TypeVar("A")
@@ -89,9 +85,20 @@ class Pointer(Generic[T]):
         if data.type is not self.type:
             raise ValueError("pointer must be the same type")
 
-        bytes_a = (ctypes.c_ubyte * sys.getsizeof(~data)) \
+        deref_a: T = ~data
+        deref_b: T = ~self
+
+        size_a: int = sys.getsizeof(deref_a)
+        size_b: int = sys.getsizeof(deref_b)
+
+        if not size_a == size_b:
+            raise MemoryError(
+                f"object is of size {size_a}, while memory allocation is {size_b}"  # noqa
+            )
+
+        bytes_a = (ctypes.c_ubyte * sys.getsizeof(deref_a)) \
             .from_address(data.address)
-        bytes_b = (ctypes.c_ubyte * sys.getsizeof(~self)) \
+        bytes_b = (ctypes.c_ubyte * sys.getsizeof(deref_b)) \
             .from_address(self.address)
 
         ctypes.memmove(bytes_b, bytes_a, len(bytes_a))
@@ -102,41 +109,12 @@ class Pointer(Generic[T]):
         return self
 
     def __add__(self, amount: int):
-        self._address += amount
-        return self
+        return Pointer(self.address + amount, self.type)
 
     def __sub__(self, amount: int):
-        self._address -= amount
-        return self
+        return Pointer(self.address - amount, self.type)
 
 
 def to_ptr(val: T) -> Pointer[T]:
     """Convert a value to a pointer."""
     return Pointer(id(val), type(val))
-
-
-def decay(func: Callable[P, T]) -> Callable[..., T]:
-    """Automatically convert values to pointers when called."""
-
-    @wraps(func)
-    def inner(*args: P.args, **kwargs: P.kwargs) -> T:
-        hints = get_type_hints(func)
-        actual: dict = {}
-        params = inspect.signature(func).parameters
-
-        # mypy is giving false positives here since it doesn't know how to
-        # handle paramspec
-        for index, key in enumerate(params):
-            if key in kwargs:  # type: ignore
-                actual[key] = kwargs[key]  # type: ignore
-            else:
-                with suppress(IndexError):
-                    actual[params[key].name] = args[index]  # type: ignore
-
-        for key, value in hints.items():
-            if (hasattr(value, "__origin__")) and (value.__origin__ is Pointer): # noqa
-                actual[key] = to_ptr(actual[key])
-
-        return func(**actual)
-
-    return inner
